#! /usr/bin/env perl
$::Revision='$ this file belongs to the Molcas repository $';

use strict;

#=======================================================================
#
# This script will create dependency files by scanning the 'Makefile'
# and the source files. Tho files are created: i) '00dependencies' which
# contains dependencies on header or include files per object file,
# ii) '00sources' which contain lists of all source and object files.
# All Fortran source files, '*.f', and all C source files, '*.c', are
# scanned for include files, '*.inc', and header files, '*.h' and
# '*.fh'. When an include files is detected, it is searched for in the
# current directory, in ${MOLCAS}/include and ${GAINC} in that order.
# Note that there have to be a configured 'Makefile' in the current
# directory since this file is used to define the include directories. 
# Note that nested includes is also resolved.
#
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
# This is the older MkDep.sh, rewritten almost one-to-one in Perl
# and with a new feature: resolving of the nested includes.
# Author of this version: Alexander Gaenko
#                         Lund University, Sweden
# Author of the original shell-scripted version:
#          Per-Olof Widmark
#          Lund University, Sweden
# Modified: Valera Veryazov, ESQC-07
#
# Written: November 2003, May 2004
# ---------------------------------------------------------------------

#-----------------------------------------------------------------------
#
# Global variables:
#
use vars qw(
 $Dep_File $Src_File $Obj_List $Version $Header $Revision
 $Molcas $Exe $Prog $IncDir  $Util
 $GotInclude $Dep $ObjDep $LibDep $Inc $TotList
 $FSrc $CSrc $IncSrc
 $PLibObj_List
 %Resolved
 %ModMap
);

#-----------------------------------------------------------------------
#
# This function initialize the procedure and open output files
# Syntax: Initialize();
#
#-----------------------------------------------------------------------
sub Initialize
{
 $Dep_File='00dependencies';
 $Src_File='00sources';
 $Obj_List='OBJ  =';
 # unlink($Dep_File,$Src_File);
 $Version=(split ' ',$Revision)[1]; # FIXME!: unreliable

 if ($Header) {
   open(DEP_FILE,">$Dep_File") || die "Initialize: can't open $Dep_File: $!";
   open(SRC_FILE,">$Src_File") || die "Initialize: can't open $Src_File: $!";
   print DEP_FILE "# \$Revision: $Version \$\n";
   print SRC_FILE "# \$Revision: $Version \$\n";
 } else {
   open(DEP_FILE,">>$Dep_File") || die "Initialize: can't open $Dep_File: $!";
   open(SRC_FILE,">>$Src_File") || die "Initialize: can't open $Src_File: $!";
 }
}
#-----------------------------------------------------------------------
#
# This function parses program arguments
# Syntax: ParseArguments()
#
#-----------------------------------------------------------------------

sub ParseArguments
{
 $Header=1;
 my $opt;
 for $opt (@ARGV) {
   my $opt=$_;
   if ($opt eq '-noheader') {
     $Header=0;
   } else {
     print "Syntax: $0 [-noheader]\n";
     exit 1;
   }
 }
}

#-----------------------------------------------------------------------
#
# This function scans the file Makefile for definitions.
# Syntax: ScanMakefile()
#
#-----------------------------------------------------------------------
sub ScanMakefile 
{
 #
 # Find Makefile
 #
 my $file='Makefile';
 if (! -r "$file" ) {
   Debug("Did NOT find file '$file'\n");
   die ("Makefile is not found! rerun configure\n");
   exit 1;
 } else {
   Debug("Did find file '$file'\n");
 }
 #
 # Determine MOLCAS include file
 #
 open(FILE,"<$file") || die "ScanMakefile(): Can't open '$file': $!";
 # MOLCAS=`grep '^MOLCAS' "$file" | awk '{print $3}'`
 #  $Molcas=(map {(split ' ',$_)[2]} grep {/^MOLCAS/} <FILE>)[-1];
 $Exe='';
 while (<FILE>) {
   my $line=$_;
   chomp $line;
   
   if ($line=~/^MOLCAS/) {
     $Molcas=(split ' ',$line)[2]; # FIXME!!! VERY unreliable!
     next;
   }
   if ($line=~/^EXE/) {
     $Exe=(split ' ',$line)[2]; # FIXME!!! VERY unreliable!
   }

#   if ($line=~/^GAINC/) {
#     $GAInc=(split ' ',$line)[2]; # FIXME!!! VERY unreliable!
#   }
 }
 close(FILE);
 
 if (!$Exe) {
     #$Exe='molcas';
   $Exe=$ENV{PWD};
   $Util=1;
 }
 $Exe=~/([^\/]*)$/; $Prog=$1; # basename($Exe)
 $Prog=~s/\.exe$//;
 $IncDir="$Molcas/src/Include";
 
 Debug(<<"~~VARS~~");
MOLCAS=$Molcas
EXE   =$Exe
PROG  =$Prog
INCDIR=$IncDir
~~VARS~~
 # That's all
}

#-----------------------------------------------------------------------
#
# This function detects and locates the 1st-level include files.
# 
# In: $filename
# Out: @array of located include files
# Global vars:
#  $Dep -- (a) space-separated list of dependencies (for makefile)
#  %Resolved -- (r) hash of already analyzed files
#  $Inc -- temporary var that has no reason to be global
#-----------------------------------------------------------------------
sub DirectlyIncludedFiles
{
 my $file=shift;
 open(FILE,"<$file") || die "IncludeFiles(): Can't open '$file': $!";

 my (%list1,%list2,%list3); # hashes (not arrays) to ensure uniqueness
 while (<FILE>) {
   my $line=$_;
   chomp($line);

   if ($line=~/^ *include/i && $line!~/=/) {
     $line=(split ' ',$line)[1]; # FIXME!!! Is it reliable?
     $line=~ tr/'"//d; # remove " and '
     $list1{$line}=1;
     next;
   }

   if ($line=~/^ *#include *</i) {
     $line=~s/.*<(.*)>.*/$1/;
     $list2{$line}=1;
     next;
   }

   if ($line=~/^ *#include *"/i) {
     $line=~s/.*"(.*)".*/$1/;
     $list3{$line}=1;
     next;
   }
   
       
 }
 close(FILE);

 my @included=();
 my $fullname;
 for $Inc (sort(keys %list1), sort(keys %list2), sort(keys %list3)) {
   # No explicit dependency on buildinfo.fh :
   #if  ($Inc eq "buildinfo.fh") {
   #     last; # FIXME! I don't understand this. Maybe, "next"?
   #  }
   
   $fullname="./$Inc";
   next if $Resolved{$fullname};
   if (-r $fullname) {
     push @included,$fullname;
     $Dep.=" ./$Inc";
     next;
   }
   
   
   $fullname="$IncDir/$Inc";
   next if $Resolved{$fullname};
   if (-r $fullname) {
     push @included,$fullname;
     $Dep.=" \${INCDIR}/$Inc";
     next;
   }


   $fullname="/usr/include/$Inc";
   if (-r $fullname) {
     next; # do nothing
   }
   
#   print "Warning: Could not locate include file '$Inc' for file '$file'\n";
 }
 
 return @included;
}
 
sub DirectlyUsedFiles
{
 my $file=shift;
 open(FILE,"<$file") || die "IncludeFiles(): Can't open '$file': $!";

 my (%list_use); # hashes (not arrays) to ensure uniqueness
 while (<FILE>) {
   my $line=$_;
   chomp($line);

   if ($line=~/^ *USE +/i) {
     $line=~s/^ *USE +//i;
     $line=~s/,.*$//;
     $list_use{lc $line}=1;
     next;
   }
       
 }
 close(FILE);

 my @included=();
 my $fullname;

 for my $Mod (sort(keys %list_use)) {
   # get actual file name containing the module
   my $Inc = $ModMap{$Mod};
   if ($Inc) {
   $Inc =~ s/\.\w+$//;
   $fullname="\${LIBDIR}/lib$Prog.a($Inc.o)";
   next if $Resolved{$fullname};
     push @included,$fullname;
   }
 }

 
 return @included;
}

#-----------------------------------------------------------------------
#
# This function detects and locates the include files, and build
# dependencies.
# In: $filename
# Out: nothing
# Globals:
#   $Dep -- (w) string with space-separated list of includes
#   %Resolved -- (w) hash of files which dependencies are already resolved
#   $ObjDep,$LibDep,$TotList -- (w) lines with dependencies
#   $GotInclude -- temporary flag var that has no reason to be global
#
#-----------------------------------------------------------------------
sub IncludeFiles 
{
 my $file=shift;
 
 my $debased=$file; $debased=~s/\.[cf]9?[05]?$//i; #FIXME shoul be .c, .f, .f90, F90, f95
 my @used=();
 my $extra='';
 my $U;
 if($file=~/\.f/i) {
 @used=DirectlyUsedFiles($file);
 
 foreach $U (@used)
 {
   $extra=$extra.' '.$U;
 }
 }
 
 $ObjDep="$debased.o : $file $extra";
 $LibDep="\${LIBDIR}/lib$Prog.a($debased.o) : $file $extra";

 $Dep='';
 %Resolved=();
 my @to_resolve=($file);
 $GotInclude=0;
 while (@to_resolve) {
   my $f=shift @to_resolve;
   $Resolved{$f}=1;
   my @includes=DirectlyIncludedFiles($f);
   $GotInclude=1 if (@includes);
   push @to_resolve,@includes;
 }
 
 $Dep=~s/^ *//;
# $TotList=' ';

## VV: possible bug - if there is no include - 00dependencies is void.
# if ($GotInclude) {
 if ($debased!~/^main$/) {
   print DEP_FILE "$ObjDep $Dep\n".
                  "$LibDep $Dep\n";
   $TotList.=" $Dep";
 }
 else
 {
   print DEP_FILE "$debased.o : $file \n";
 
 }
}

#-----------------------------------------------------------------------
#
# This function produce the Fortran source list
# Syntax: FortranSource()
#
#-----------------------------------------------------------------------
sub FortranSource
{
 $FSrc="FSRC =";
 my $n;
 my $file;
 my @extlist=('.f','.f90','.F90','.f95','.F95');
 my $ntot = 0;
 foreach my $ext (@extlist)
 {
 $n=0;
 for $file (<*$ext>) {
   if ($file ne "main$ext" && $file ne 'tmp.f') {
     if ($n==5) {
       print SRC_FILE "$FSrc \\\n";
       $FSrc="      ";
       $n=0;
     }
     $FSrc.=" $file";
     $n++;
     $ntot++;
   }
 }
 if ($n!=0) {
   $Obj_List.=" \${FSRC:$ext=.o}";
 }
 }
 if ($ntot!=0) {
   print SRC_FILE "$FSrc\n";
 }
}

# map module names to file names
# for resolving 'USE' dependencies
sub FortranModules
{
 my @extlist=('.f','.f90','.F90','.f95','.F95');
 foreach my $ext (@extlist)
 {
  for my $file (<*$ext>) {
   open(FILE,"<$file") || die "FortranModules(): Can't open '$file': $!";
   while (<FILE>) {
    my $line=$_;
    chomp($line);

    if ($line =~ /^ *module +/i) {
     $line=~s/^ *module +//i;
     $ModMap{lc $line} = $file;
     next;
    }

   }
   close(FILE);
  }
 }
}

#-----------------------------------------------------------------------
#
# This function produce the C source list
# Syntax: CSource()
#
#-----------------------------------------------------------------------
sub CSource 
{
 $CSrc="CSRC =";
 my $n=0;
 my $file;
 for $file (<*.c>) {
   if ($file ne 'main.c') {
     if ($n==5) {
       print SRC_FILE "$CSrc \\\n";
       $CSrc="      ";
       $n=0;
     }
     $CSrc.=" $file";
     $n++;
   }
 }
 if ($n!=0) {
   print SRC_FILE "$CSrc\n";
   $Obj_List.=" \${CSRC:.c=.o}";
 }
}

#-----------------------------------------------------------------------
#
# This function produce the include source list
# Syntax: IncSource()
#
#-----------------------------------------------------------------------
sub IncSource
{
 my %inc_list;
 my $file;
 # [grumble: silly and inefficient translation from shell script]
 if(defined($TotList))
 {
 for $file (split ' ',$TotList) {
   next if $file=~/buildinfo\.fh/;
   $inc_list{$file}=1;
 }
 $IncSrc="INCSRC =";
 my $n=0;
 for $file (sort keys %inc_list) {
   if ($n==5) {
     print SRC_FILE "$IncSrc \\\n";
     $IncSrc="      ";
     $n=0;
   }
   $IncSrc.=" $file";
   $n++;
 }
 if ($n!=0) {
   print SRC_FILE "$IncSrc\n";
 }
 }
}

#-----------------------------------------------------------------------
#
# This function produce the PLIB object list
# Syntax: PLibObjList()
#
#-----------------------------------------------------------------------
sub PLibObjList
{
 $PLibObj_List="PLIBOBJ =";
 my $n=0;
 my $file;
 my @extlist=('.f','.f90','.F90','.f95','.F95','.c');

 foreach my $ext (@extlist)
 {
 for $file (<*$ext>) {
   if ($file ne "main$ext" && $file ne 'main.c') {
     if ($n==1) {
       print SRC_FILE "$PLibObj_List \\\n";
       $PLibObj_List="      ";
       $n=0;
     }
     $file=~s/\.[cf]9?[05]?$//i;
     $PLibObj_List.=" \${LIBDIR}/lib$Prog.a($file.o)";
     $n++;
   }
 }
 }
 if ($n!=0) {
   print SRC_FILE "$PLibObj_List\n";
 }
} 

#=======================================================================
#
# Debug print functon
#
#=======================================================================
sub Debug
{
# print "===Debug:===\n@_\n=========\n";
}


#=======================================================================
#
# Body
#
#=======================================================================
ParseArguments();
Initialize();
ScanMakefile();
FortranSource();
FortranModules();
CSource();
print SRC_FILE "$Obj_List\n";
PLibObjList();

my $source;
my @extlist=('.f','.f90','.F90','.f95','.F95','.c');
foreach my $ext (@extlist)
{
 for $source (sort <*$ext>) {
   IncludeFiles($source);
 }
}
IncSource();
exit 0;

