#! /usr/bin/env perl
# check_shar.plx $ this file belongs to the Molcas repository $

use strict;
use warnings;

my $required_version = $ARGV[0] || die "Usage: check_shar.plx \$required_version\n";


if ($required_version !~ /^\d+\.\d+\.\d+$/) {
    die "The required version must be in the format: 4.2.1\n";
}

my $shar_exe   = `which shar 2>/dev/null`;
my $gshar_exe  = `which gshar 2>/dev/null`;
chomp($shar_exe);
chomp($gshar_exe);

my $gshar_ok = 0;
my $gshar_version = 0;
if (-x $gshar_exe) {
   my $version = `$gshar_exe --version 2>/dev/null`;
      {
       $version=~s/\b\w\(\)//g;
       $gshar_version = $version;
      if ($gshar_version ge $required_version) {
         $gshar_ok = $gshar_version;
      }
   }
}

my $shar_ok = 0;
my $shar_version = 0;
if (-x $shar_exe) {
   my $version = `$shar_exe --version 2>/dev/null`;
   if($version =~ /^.+\bGNU\b/)
      {
       $version=~s/\b\w\(\)//g;
      $shar_version = $version;
      if ($shar_version ge $required_version) {
         $shar_ok = $shar_version;
      }
   }
}

my $shar_to_use = 0;
if ($gshar_ok && $shar_ok) {
   if ($gshar_ok ge $shar_ok) {
      $shar_to_use = $gshar_exe;
   } else {
      $shar_to_use = $shar_exe;
   }
} elsif ($gshar_ok) {
   $shar_to_use = $gshar_exe;
} elsif ($shar_ok) {
   $shar_to_use = $shar_exe;
} else {
   warn "--- No usable GNU shar found.\n--- You must have GNU shar, version number at least $required_version\n";
   if ($gshar_version || $shar_version) {
      warn "--- You have the following version(s) installed:\n";
      warn "--- $gshar_exe version $gshar_version\n" if ($gshar_version);
      warn "--- $shar_exe version $shar_version\n" if ($shar_version);
   }
   die "No usable GNU shar found\n";
}

print "$shar_to_use";
