#!/usr/bin/python 
# $ this file belongs to the Molcas repository $
# from Standard Python 
"""
Written by Asbjoern Holt in 2006. Feel free to change/fix/improve/destroy
"""

import sys, os, glob, copy, tempfile, string

try:
    from Tkinter import *
    from tkMessageBox import *
    from tkFileDialog import *
    from tkSimpleDialog import *
except ImportError:
    print "Tkinter not installed. Graphical installation terminated. For installation of Tkinter, please consult http://www.python.org/"
    sys.exit(1)
    

# Some layout settings
labelfont = ('curier',12)
labelfont2 = ('times',14)

# from Yattagawa
class BasicScrollText2(Frame):
    def __init__(self, parent=None, text='', file=None):
        Frame.__init__(self, parent)
        self.pack(expand=YES, fill=BOTH)
        self.lastfind = None
        self.makewidgets()
        self.settext(text, file)
        self.file = file
        
    def makewidgets(self):
        sbar = Scrollbar(self)
        sbarx = Scrollbar(self)
                
        text = Text(self, relief=SUNKEN, font=("Times", "10") )
        sbar.config(command=text.yview)        
        sbarx.config(command=text.xview,  orient=HORIZONTAL)
        text.config(xscrollcommand=sbarx.set, yscrollcommand=sbar.set, height=5)
        sbar.pack(side=RIGHT, fill=Y)
        sbarx.pack(side=BOTTOM, fill=X)
        text.pack(side=LEFT, expand=YES, fill=BOTH)
        self.text=text
        frm = Frame(self)
        frm.pack(side=LEFT, expand=NO, fill=BOTH)
        
    def settext(self, text='', file=None):
        if file:
            self.file=file
            text1 = open(file, 'r').readlines() 
            text = '' 
            for i in text1: 
                if i[0]=='#': pass 
                else: text=text+i 
 
        self.text.delete('1.0', END) 
        self.text.insert('1.0', text) 
        self.text.mark_set(INSERT, '1.0') 
 
#### new classes 
## A small container 
class Code: 
    def __init__(self, name='', guess='', descr=''): 
        self.name = name 
        self.guess = guess 
        self.descr = descr 
 
    def show(self): 
        print "%s %s %s" % (self.name, self.guess, self.descr) 
 
# A Class containing info about the compilers 
class Compiler: 
    def __init__(self, name='', loc='', flag=''): 
        self.name = name 
        self.loc  = loc 
        self.flag = flag 
 
    def show(self): 
        print "Name: %s \t loc: %s \t flag: %s \n" % (self.name, self.loc, self.flag) 
         
# A "new" checkbutton to enable help 
class myCheckbutton(Checkbutton): 
 
    def __init__(self, parent=None,text='', variable=None, descr=''): 
        Checkbutton.__init__(self, parent,text=text, variable=variable) 
        self.descr=descr 
 
    def show(self): 
        print self.descr 


# A waitbar
class WaitBar(Frame):

    def __init__(self, parent=None, nr=100):
        Frame.__init__(self, parent)

        self.co=0
        self.count = 0.0
        self.nr = nr
        self.dr = self.nr/100.0
        self.var = IntVar()
        self.buildWaitBar()

    def buildWaitBar(self):
        self.Bar = Scale(self, from_=0, to=100,  resolution=1, orient='horizontal', variable=self.var)
        self.Bar.config(state=DISABLED)
        self.Bar.pack(fill=X, expand=YES)

    def MoveBar(self):
        self.co+=1
        self.count=self.count+self.dr
        self.var.set(self.co)

    def setvar(self, nr, fin=None):

        if not fin:
            a = 100-int(100*(self.nr-nr)/self.nr)
        else:
            a = 100

        self.var.set(a)
        
#### the main gui class 
class tkMolcasConfig: 
    def __init__(self): 
        self.main=Tk() 
        self.main.title('Molcas Configure') 
        # w, h = self.main.winfo_screenwidth(), self.main.winfo_screenheight() 
        # self.main.overrideredirect(1) 
        # self.main.geometry("%dx%d+0+0" % (w, h)) 

        #self.main.maxsize(width=800, height=500)
        self.main.minsize(width=500, height=500)

        self.GotoBLAS = None
 
        self.widget=Frame(self.main) 
 
        self.widget.config(bd=1, relief=SUNKEN) 
        self.widget.pack(expand=YES, fill=BOTH) 
        self.main.bind('<Escape>', self.escp ) # Exit on escape (without doing anything) 
        self.widget.focus() 
        self.widget.config(cursor='arrow')
        
        self.buildUpperWidget()
        self.getsetupdate() 
         
        # Define configure dictionary 
        self.configDict = {"compiler": '',  
                           "noprompt": ' -noprompt ', 
                           "mpi"     : self.MPI,
                           "blas"    : "",
                           "blasDir" : ""} 
        self.buildFirstSetup() 
 

        self.IWasInSelectPackages=0
        self.DoIWantToUseBLAS=0
        self.myBlasSelection=0
 
        ## Slogans 
        self.slogannumber=0 
        # Read slogan-list 
        self.sloganList=[] 
        for i in open('data/quote.txt','r').readlines()[1:]: 
            self.sloganList.append(i[:-1]) 
        # Start showing slogans 
        self.onSlogans() 
 
    def buildUpperWidget(self): 
        self.Name = Label(self.widget, font=('curier', 16, 'bold'), text='Molcas Configure') 
        self.Name.pack(expand=NO, fill=BOTH) 
 
        # display of textfile 
        self.confframe = Frame(self.widget) 
        self.input = BasicScrollText2(self.confframe, file='data/welcome.txt') 
        self.input.pack(side=LEFT) 
 
        # display Logo 
        self.img = PhotoImage(file='data/molcas_logo300.gif') 
         
        can = Canvas(self.confframe) 
        can.config(width=self.img.width(), height=self.img.height()) 
        can.create_image(0,0,image=self.img, anchor = NW) 
        can.pack(side=RIGHT) 
 
        # pack top 
        self.confframe.config(relief=SUNKEN, borderwidth=1) 
        self.confframe.pack(side=TOP, anchor=N, expand=YES, fill=BOTH) 
 
        # Display Slogans 
        self.slogans = Label(self.widget, text='', relief=RIDGE) 
        self.slogans.pack(side=TOP) 
        Label(self.confframe, text='').pack(side=BOTTOM) 

    def onSelectBlasDir(self):
        if self.blasInt.get()=="1":
            self.GotoBLASButton.config(state=DISABLED)
        elif self.blasInt.get()=="2":
            self.GotoBLASButton.config(state=DISABLED)
        else:
            self.GotoBLASButton.config(state=NORMAL)
        self.GotoBLASButton.pack()
        
    def buildGoToBlasWidget(self):
        self.GotoBLASSetup = Frame(self.widget)
        self.GotoBLASSetup.pack(side=TOP, anchor=W)

        self.DoBlas = IntVar()
        abutton = Checkbutton(self.GotoBLASSetup, text='Are you going to use GotoBLAS?', relief = RIDGE, variable=self.DoBlas, command=self.onBlasClick)
        abutton.pack(side=TOP,anchor=W,expand=YES)
        if self.DoIWantToUseBLAS!=0:
            abutton.toggle()
            
        self.blasButtonFrame = Frame(self.GotoBLASSetup)
        self.blasButtonFrame.pack()
        Label(self.blasButtonFrame, text="Use;").pack(side=TOP, anchor=W)
        self.blasInt = StringVar()
        self.c1=Radiobutton(self.blasButtonFrame, text='MOLCAS-BLAS', variable=self.blasInt, value="1", command=self.onSelectBlasDir)
        self.c1.pack(side=TOP, anchor=W)
        self.c2=Radiobutton(self.blasButtonFrame, text="The lapack package", variable=self.blasInt, value="2", command=self.onSelectBlasDir)
        self.c2.pack(side=TOP, anchor=W)
        self.c3=Radiobutton(self.blasButtonFrame, text="GotoBLAS", variable=self.blasInt, value='libgoto.a', command=self.onSelectBlasDir)
        self.c3.pack(side=TOP, anchor=W)
        self.c4=Radiobutton(self.blasButtonFrame, text="Atlas", variable=self.blasInt, value="libatlas.a", command=self.onSelectBlasDir)
        self.c4.pack(side=TOP, anchor=W)
        self.c5=Radiobutton(self.blasButtonFrame, text="MKL", variable=self.blasInt, value="libmkl_lapack.a", command=self.onSelectBlasDir)
        self.c5.pack(side=TOP, anchor=W)
        self.c6=Radiobutton(self.blasButtonFrame, text="ACML", variable=self.blasInt, value="libacml.a", command=self.onSelectBlasDir)
        self.c6.pack(side=TOP, anchor=W)
        self.GotoBLASButton = Button(self.blasButtonFrame, command=self.GetGotoBLAS, text='Locate the Selected BLAS Directory')
        self.GotoBLASButton.pack(side=LEFT, anchor=W, fill=X, expand=YES)
        self.blasLabel = Label(self.blasButtonFrame,text="")
        self.blasLabel.pack()

        if self.myBlasSelection==0 or self.myBlasSelection=="1":
            self.c1.select()
        elif self.myBlasSelection=="2":
            self.c2.select()
        elif self.myBlasSelection=='libgoto.a':
            self.c3.select()            
        elif self.myBlasSelection=="libatlas.a":
            self.c4.select()
        elif self.myBlasSelection=="libmkl_lapack.a":
            self.c5.select()
        elif self.myBlasSelection=="libacml.a":
            self.c6.select()
            
        self.buttonframe = Frame(self.widget)
        self.buttonframe.pack(side=BOTTOM, anchor=S, fill=X, expand=YES)
        Label(self.buttonframe,text='').pack(side=TOP)
        self.exitButton=Button(self.buttonframe, command=self.onBack3, text='Back')
        self.exitButton.pack(side=RIGHT, anchor=E, fill=X, expand=YES)
        self.installButton=Button(self.buttonframe, command=self.GotoBLASNext, text='Next')
        self.installButton.pack(side=LEFT, anchor=W, fill=X, expand=YES)
        self.onBlasClick()
        
    def onBlasClick(self):
        if self.DoBlas.get():
            self.GotoBLASButton.config(state=DISABLED)
            self.c1.config(state=NORMAL)
            self.c2.config(state=NORMAL)
            self.c3.config(state=NORMAL)
            self.c4.config(state=NORMAL)
            self.c5.config(state=NORMAL)
            self.c6.config(state=NORMAL)
            self.onSelectBlasDir()
        else:
            self.GotoBLASButton.config(state=DISABLED)
            self.c1.config(state=DISABLED)
            self.c2.config(state=DISABLED)
            self.c3.config(state=DISABLED)
            self.c4.config(state=DISABLED)
            self.c5.config(state=DISABLED)
            self.c6.config(state=DISABLED)


        self.GotoBLASButton.pack()
        self.c1.pack()
        self.c2.pack()
        self.c3.pack()
        self.c4.pack()
        self.c5.pack()
        self.c6.pack()

    def GetGotoBLAS(self):
        self.GotoBLAS = askdirectory()
        if self.GotoBLAS:
            self.blasLabel.config(text=self.GotoBLAS)
            self.blasLabel.pack()
        
    def GotoBLASNext(self):
        self.blasString = ""
        self.blasDir    = ""
        self.DoIWantToUseBLAS=self.DoBlas.get()
        self.myBlasSelection=self.blasInt.get()
        if self.DoBlas.get():
            self.widget.config(cursor='watch')
            self.installButton.config(state=DISABLED)
            self.exitButton.config(state=DISABLED)
            
            self.main.update()

            self.myBlasSelection=self.blasInt.get()
            try:
                if self.blasInt.get()!="1":
                    if self.blasInt.get()!="2":
                        open( os.path.join(self.GotoBLAS, self.blasInt.get()),'r').read()
                        self.configDict["blasDir"]  = self.GotoBLAS
                
                if   self.blasInt.get()=="1":
                    self.configDict["blas"] = ' '
                elif self.blasInt.get()=="2":
                    self.configDict["blas"] =  " -blas lapack "
                elif self.blasInt.get()=='libgoto.a':
                    self.configDict["blas"] =  " -blas GotoBLAS"
                elif self.blasInt.get()=="libatlas.a":
                    self.configDict["blas"] =  " -blas Atlas"
                elif self.blasInt.get()=="libmkl_lapack.a":
                    self.configDict["blas"] =  " -blas MKL"
                elif self.blasInt.get()=="libacml.a":
                    self.configDict["blas"] =  " -blas ACML"
                


            except:
                showerror('WARNING','You failed to spesify a correct directory. \n Please go back and spesify a correct directory')
                self.configDict["blas"]     = ""
                self.configDict["blasDir"]  = ""
                self.widget.config(cursor='arrow')
                self.installButton.config(state=ACTIVE)
                self.exitButton.config(state=ACTIVE)
                return 
        else:
            self.configDict["blas"]     = ""
            self.configDict["blasDir"]  = ""       

        self.GotoBLASSetup.destroy()
        self.buttonframe.destroy()       
        self.widget.config(cursor='arrow')
        self.buildConfigureSetup()
        
    def buildSetupWidget(self):
        
        self.lowerFrame = Frame(self.widget) 
        self.lowerFrame.pack(side=TOP, anchor=W, fill=X) 
 
        # Label for help 
        self.help = Label(self.lowerFrame, text='') 
        self.help.pack(side=TOP, fill=X, expand=YES) 
         
        self.var = [] # used to store the values of the checkboxes 
          
        # the checkboxes 
        Label(self.lowerFrame, text='Mark Molcas codes you want to use').pack(side=TOP, anchor=W) 
        self.rbframe = Frame(self.lowerFrame, relief=SUNKEN, bd=1) 
 
        self.ListOfBoxes=[] 
        tmpList = [] 
        cols=-1 
        rows=0 
        for key in self.packageList: 
            # Build Checkbuttons 
            if key.guess!='# guess=hide': 
                tmpList.append(copy.deepcopy(key)) 
                cols=cols+1 
                var = IntVar() 
                self.ListOfBoxes.append( myCheckbutton(self.rbframe, text=key.name, variable=var, descr=key.descr) ) 
                
                if key.guess=='# guess=yes': self.ListOfBoxes[-1].toggle() # set a tick in the box 
                self.ListOfBoxes[-1].grid(column=cols,row=rows, sticky=W) 
                self.var.append(var) 
                self.rbframe.columnconfigure(cols, weight=1) 
            else: self.var.append(None) 
             
            if cols==3: 
                cols=-1 
                rows=rows+1 
 
        # Add some events 
        for i in range(len(self.ListOfBoxes)): 
            self.ListOfBoxes[i].bind('<Enter>', self.onMouseEnter ) 
            self.ListOfBoxes[i].bind('<Leave>', self.onMouseLeave )  
             
        self.rbframe.pack(side=TOP,anchor=N,expand=YES, fill=X) 
         
        ################## 
        Label(self.lowerFrame, text='Selected additional packages').pack(side=TOP, anchor=W) 
        self.packageframe = Frame(self.lowerFrame, relief=SUNKEN, bd=1) 
        self.pvar = [] 
        self.aList = findPackages() 
        if self.aList: 
            cols=-1 
            rows=0 
            for i in self.aList: 
                # Build Checkbuttons 
                cols=cols+1 
                var = IntVar() 
                c = myCheckbutton(self.packageframe, text=i, variable=var) 
                c.grid(column=cols,row=rows, sticky=W)
                if self.firstSetupVar.get()==2:
                    c.toggle()
                self.pvar.append(var) 
                self.packageframe.columnconfigure(cols, weight=1) 
                 
                if cols==3: 
                    cols=-1 
                    rows=rows+1 
        else: 
            Label(self.packageframe,text='No Packages found').grid(column=0,row=0, sticky=W) 
            self.rbframe.columnconfigure(0, weight=1) 
         
        self.packageframe.pack(side=TOP,anchor=N,expand=YES, fill=X) 
 
        # Buttons 
        self.buttonframe = Frame(self.widget) 
        self.buttonframe.pack(side=BOTTOM, anchor=S, fill=X, expand=YES) 
        Label(self.buttonframe,text='').pack(side=TOP) 
        self.exitButton=Button(self.buttonframe, command=self.onBack0, text='Back') 
        self.exitButton.pack(side=RIGHT, anchor=E, fill=X, expand=YES) 
        self.installButton=Button(self.buttonframe, command=self.OK, text='Next') 
        self.installButton.pack(side=LEFT, anchor=W, fill=X, expand=YES) 

    def buildFirstSetup(self):
        self.IWasInSelectPackages=0
        self.firstSetup = Frame(self.widget)
        self.firstSetup.pack(side=TOP, anchor=W)
        Label(self.firstSetup, text='How do you want to set up your system').pack(side=TOP, anchor=W)
        self.firstSetupVar = IntVar()
        c1=Radiobutton(self.firstSetup, text='Minimal', variable=self.firstSetupVar, value=1)
        c1.pack(side=TOP, anchor=W)
        c2=Radiobutton(self.firstSetup, text='Full', variable=self.firstSetupVar, value=2)
        c2.pack(side=TOP, anchor=W)
        c3=Radiobutton(self.firstSetup, text='Select manually', variable=self.firstSetupVar, value=3)
        c3.pack(side=TOP, anchor=W)
        c2.select()
        
        self.buttonframe = Frame(self.widget)
        self.buttonframe.pack(side=BOTTOM, anchor=S, fill=X, expand=YES)
        Label(self.buttonframe,text='').pack(side=TOP)
        self.exitButton=Button(self.buttonframe, command=self.escp, text='Exit')
        self.exitButton.pack(side=RIGHT, anchor=E, fill=X, expand=YES)
        self.installButton=Button(self.buttonframe, command=self.onFirst, text='Next')
        self.installButton.pack(side=LEFT, anchor=W, fill=X, expand=YES)
 
    def buildConfigureSetup(self): 
        try:
            # Compiler section 
            self.compilerframe = Frame(self.widget) 
            self.compilerframe.pack(side=TOP, expand=YES, fill=X) 

            fastndb = Frame(self.compilerframe)
            fastndb.pack(side=TOP, anchor=W)
            self.fastnormaldebug = IntVar()
            c1 = Radiobutton(fastndb, text='Fast',   value=1, variable=self.fastnormaldebug, command=self.onCompiler)
            c1.pack(side=LEFT)
            c2 = Radiobutton(fastndb, text='Normal', value=2, variable=self.fastnormaldebug, command=self.onCompiler)
            c2.pack(side=LEFT)
            c3 = Radiobutton(fastndb, text='Debug',  value=3, variable=self.fastnormaldebug, command=self.onCompiler)
            c3.pack(side=LEFT)
            c2.select()

            self.compilerVar = IntVar() 
            Label(self.compilerframe,text='Select compiler;').pack(anchor=W)  
            self.compilerListFrame = Frame(self.compilerframe) 
            self.compilerListFrame.pack(side=LEFT, anchor=W) 
            Label(self.compilerListFrame, text='Compiler').grid(column=0,row=0, sticky=W) 
            Label(self.compilerListFrame, text='Location').grid(column=1,row=0, sticky=W) 
            for i in range(len(self.compilerList)): 
                txt = self.compilerList[i].name #os.path.split(i)[-1] 
                c=Radiobutton(self.compilerListFrame, text=txt, variable=self.compilerVar, command=self.onCompiler, value=i) 
                c.grid(column=0,row=i+1, sticky=W) 
                txt2 = self.compilerList[i].loc 
                Label(self.compilerListFrame, text=txt2).grid(column=1,row=i+1, sticky=W) 
            c.select() 

            # User selected compiler 
            #        c=Radiobutton(self.compilerListFrame, text='Select Other', variable=self.compilerVar, value=i+1, command=self.onUserSelectedCompiler) 
            #        c.grid(column=0,row=i+2, sticky=W) 
            #        self.comSelf=Label(self.compilerListFrame, text='') 
            #        self.comSelf.grid(column=1,row=i+2, sticky=W) 

            # Configure entry 
            self.configframe = Frame(self.widget)
            if self.pathList:
                self.UserPath = os.getcwd()
                Label(self.configframe, text='Select Path').pack(side=TOP, anchor=W)
                self.pathVar = StringVar()
                for i in self.pathList:
                    c=Radiobutton(self.configframe, text=i, variable=self.pathVar, value=i, command=self.onUserPath)
                    c.pack(side=TOP, anchor=W)
                c.select()
                subFrame = Frame(self.configframe)
                self.UserPathButton = Radiobutton(subFrame, text='Select other path', variable=self.pathVar, value=self.UserPath, command=lambda: self.onUserPath(usr=1))
                self.UserPathButton.pack(side=LEFT, anchor=W)
                self.pathLabel=Label(subFrame,text=self.UserPath)
                self.pathLabel.pack(side=RIGHT, anchor=E)
                subFrame.pack(side=TOP, anchor=W)

            Label(self.configframe,text='').pack(side=TOP) 
            self.configframe.pack(side=TOP, expand=YES, fill=X) 
            self.configlabel = Label(self.configframe, text='Current configure setting') 
            self.configlabel.pack(anchor=W) 
            self.config = Entry(self.configframe) 
            self.config.pack(expand=YES, fill=X) 
            self.config.insert(0,'./configure -noprompt') 
            self.onCompiler() 

            # Ok and cancel button 
            self.buttonframe = Frame(self.widget) 
            self.buttonframe.pack(side=BOTTOM, anchor=S, fill=X, expand=YES) 
            Label(self.buttonframe,text='').pack(side=TOP) 
            self.configButton=Button(self.buttonframe, command=self.onConfigure, text='Run configure') 
            self.configButton.pack(side=LEFT, fill=X, expand=YES) 
            self.back1Button=Button(self.buttonframe, command=self.onBack1, text='Back') 
            self.back1Button.pack(side=RIGHT, anchor=W,fill=X, expand=YES)
        
        except:
            sys.exit(1)
         
    def buildMakeWidget(self):

        
        os.system('ls src/* | wc -l > noentallbare_omhundreeralltingglemt.txt')
        nr =float(open('noentallbare_omhundreeralltingglemt.txt','r').read())
        os.remove('noentallbare_omhundreeralltingglemt.txt')
        
        self.buttonframe = Frame(self.widget) 
        self.buttonframe.pack(side=BOTTOM, anchor=S, fill=X, expand=YES)
        Label(self.buttonframe,text='Percent compiled').pack(side=TOP) 
        self.waitbar = WaitBar(self.buttonframe, nr)
        self.waitbar.pack(side=TOP, fill=X, expand=YES)
        Label(self.buttonframe,text='').pack(side=TOP) 
        self.makeButton=Button(self.buttonframe, command=self.onMake, text='Run make') 
        self.makeButton.pack(side=LEFT, fill=X, expand=YES)
        self.back2Button=Button(self.buttonframe, command=self.onBack2, text='Back') 
        self.back2Button.pack(side=LEFT, anchor=W, fill=X, expand=YES)  
        self.exitButton=Button(self.buttonframe, command=self.escp, text='Exit') 
        self.exitButton.pack(side=RIGHT, anchor=E, fill=X, expand=YES) 
 
    def getsetupdate(self): 
        # find info about programs 
        self.packageList = findDef() 

        #  open('setup.lst','a').write("WARN_MULTI='TEST'")

        self.setupData = open('setup.lst','r').readlines() 
 
        self.compilers    = [] 
        self.compilerList = [] 
        self.pathList     = []
        for i in self.setupData: 
 
            # Compiler Info 
            if string.split(i,'=')[0] == 'COMP_NAME': # Name 
                self.compilerList.append(Compiler(name=string.split(i,'=')[1][:-1])) 
                self.compilers.append(string.split(i,'=')[1][:-1]) 
            elif string.split(i,'=')[0] == 'COMP_FILE': # Location 
                self.compilerList[-1].loc=string.split(i,'=')[1][:-1] 
            elif string.split(i,'=')[0] == 'COMP_FLAG': # Flag 
                if len( string.split(i,'='))<=2: 
                    self.compilerList[-1].flag='' 
                else: 
                    self.compilerList[-1].flag=string.split(i,'=')[1]+' '+string.split(i,'=')[-1][:-1] 
                 
            # Other info
            elif string.split(i,'=')[0] == 'PATH':
                self.pathList.append(string.split(i,'=')[1][:-1])
            elif string.split(i,'=')[0] == 'OS': 
                self.OS =  string.split(i,'=')[1][:-1] 
            elif string.split(i,'=')[0] =='MPI': 
                self.MPI = string.split(i,'=')[1][:-1] 

            if string.split(i,'=')[0] == 'WARN_MULTI':
                err = string.join(string.split(i,'=')[1:])
                err2 = string.join(["Warning! More than one executable for ",err," has been found in your PATH Only the first one will be used!"])
                showwarning('Warning!', err2)
                

    def onFirst(self):
        nr = self.firstSetupVar.get()
        
        self.firstSetup.destroy()
        self.buttonframe.destroy()
        if   nr==1:
            self.buildSetupWidget()
            self.OK()            
        elif nr==2:
            for i in range(len(self.packageList)):
                self.packageList[i].guess='# guess=yes'
            self.buildSetupWidget()
            self.OK()
        elif nr==3:
            self.IWasInSelectPackages=1
            self.buildSetupWidget()
            
    def onUserPath(self, usr=None):
        if usr:
            tmp = askdirectory()
            if tmp:
                self.UserPath=tmp
                
                self.UserPathButton.config(value=self.UserPath)
                self.UserPathButton.pack()
                self.pathLabel.config(text=tmp)
                self.pathLabel.pack()
            else:
                self.UserPathButton.config(value=self.UserPath)
                self.UserPathButton.pack()
                self.pathLabel.config(text=self.UserPath)
                self.pathLabel.pack()
            self.UserPathButton.select()

        self.onBuildConfigure()        

    def onBack0(self):
        self.lowerFrame.destroy() 
        self.buttonframe.destroy() 
        self.buildFirstSetup()
        
    def onBack1(self): 
        self.configframe.destroy() 
        self.compilerframe.destroy() 
        self.buttonframe.destroy() 
        self.buildGoToBlasWidget()

    def onBack2(self):
        self.buttonframe.destroy() 
        self.buildConfigureSetup() 

    def onBack3(self): 
        self.GotoBLASSetup.destroy() 
        self.blasButtonFrame.destroy() 
        self.buttonframe.destroy()
        if self.IWasInSelectPackages==0:
            self.buildFirstSetup()
        else:
            self.buildSetupWidget()
        
    def onConfigure(self): 
        try: 
            # remove old symbols file 
            try: os.remove('Symbols') 
            except: pass 
             
            self.main.unbind('<Escape>') 
            self.configButton.config(state=DISABLED) 
            self.back1Button.config(state=DISABLED) 
            self.textBeforeConfig=self.input.text.get('1.0', END+'-1c') 
            self.configfile = os.path.join(os.getcwd(),'out.config.pyConfig')
            self.configerrfile = os.path.join(os.getcwd(), 'err.config.pyConfig')
            os.system('echo "Started with pyConfig" >'+self.configfile)
            confScript = open(os.path.join(os.getcwd(),'confScript.pyConf'),'w')
            if self.GotoBLAS: 
                confScript.write("export XLIB='-L"+self.GotoBLAS+" -lgoto'; \n")
            confScript.write(self.config.get()+'>>'+self.configfile+' 2>'+self.configerrfile+';\n')
            confScript.write('echo $?>'+os.path.join(os.getcwd(),'config_return.pyConf'))
            confScript.close()
            os.system("'' >"+os.path.join(os.getcwd(),'config_return.pyConf'))
            os.system('chmod u+x '+os.path.join(os.getcwd(),'confScript.pyConf')+' 2>'+self.configerrfile)
            os.system(''+os.path.join(os.getcwd(),'confScript.pyConf')+' 2>'+self.configerrfile+'&') 
            self.onDoCheckConfigure()
            self.widget.config(cursor='watch')
        except: 
            sys.exit(1) 
            
    def onCompiler(self, user=None): 

     
        # set the compiler flag 
        if not user: 
            self.configDict["compiler"] = ' '+self.compilerList[self.compilerVar.get()].flag 
        else: 
            self.configDict["compiler"] = ' -compiler='+user.flag 
             
        self.onBuildConfigure() 
 
    def onUserSelectedCompiler(self): 
        # User selected compiler 
        a = askopenfilename() 
        if not a: a='' 
        self.userCompiler = Compiler(name=a, loc=a, flag=a) 
        self.onCompiler(user=self.userCompiler) 
        self.comSelf.config(text=self.userCompiler.loc) 
        self.comSelf.update() 
         
 
    def onBuildConfigure(self): 
        self.config.delete(0,END) 
        conf = './configure ' 
        for i,j in self.configDict.items(): 
            if j: 
                conf=conf+j 
                
        # add path
        if self.pathList:
            conf = conf+' -path '+self.pathVar.get()

        # add debug/fast/normal flag
        if self.fastnormaldebug.get()==1:
            conf = conf+' -fast'
        elif self.fastnormaldebug.get()==2:
            pass
        elif self.fastnormaldebug.get()==3:
            conf = conf+' -debug'


        # if self.configDict["blas"]:
#             conf = conf + ' -blas '+self.configDict["blas"]
#             if  self.configDict["blas"]!='MOLCAS BLAS' or  self.configDict["blas"]!="lapack package":
#                 if self.configDict["blasDir"]!=None:
#                     conf = conf + " -blas_dir " + self.configDict["blasDir"]

        self.config.insert(0,conf) 
         
    def onDoCheckConfigure(self): 
        tmpfile = os.path.join(os.getcwd(),'checkConfig.pyConfig')#tempfile.mkstemp()[-1] 
        text = open(self.configfile,'r').read() 
        open(tmpfile,'w').write(self.textBeforeConfig) 
        open(tmpfile,'a').write(text) 
        self.input.settext(open(tmpfile,'r').read()) 
        self.input.text.see(END) 
        self.main.update() 
        os.remove(tmpfile) 
 
        text=[] 
        text=open(self.configfile,'r').readlines()

        bool1 = 1 
        bool2 = None 

        if not 'Configuration completed.\n' in text: 
            bool1 = None

        if open(os.path.join(os.getcwd(),'config_return.pyConf'),'r').read()!='': 
            bool2=1
            for i in open(os.path.join(os.getcwd(),'config_return.pyConf'),'r').readlines(): 
                if '0' in string.split(i): 
                    bool2 = None

        if bool2: 
            tmpfile = os.path.join(os.getcwd(),'checkConfig.pyConfig')#tempfile.mkstemp()[-1] 
            text2 = open(self.configfile,'r').read() 
            text = open(self.configerrfile,'r').read() 
            open(tmpfile,'w').write(self.textBeforeConfig) 
            open(tmpfile,'a').write(text2) 
            open(tmpfile,'a').write(text) 
            self.input.settext(open(tmpfile,'r').read())            
            self.input.text.see(END) 

            self.main.update() 
            os.remove(tmpfile)
            
            # report error 
            if askyesno('Warning!','Something went wrong when building Molcas. Do you want to exit into text-mode?'): 
                sys.exit(1)
        else:
            if not bool1:
                self.main.after(1000, self.onDoCheckConfigure) 

        if bool1 or bool2:
            os.remove(self.configfile) 
            os.remove(self.configerrfile)
            self.configframe.destroy() 
            self.compilerframe.destroy() 
            self.buttonframe.destroy() 
             
            self.main.bind('<Escape>', self.escp ) 

            os.remove(os.path.join(os.getcwd(),'config_return.pyConf'))
            os.remove('confScript.pyConf')
            self.widget.config(cursor='arrow')
            self.buildMakeWidget() 
             
    def onMake(self): 
        try: 
            self.main.unbind('<Escape>') 
            self.makeButton.config(state=DISABLED) 
            self.back2Button.config(state=DISABLED) 
            self.exitButton.config(state=DISABLED) 
             
            self.makefile = os.path.join(os.getcwd(),'make.pyConf') 
            self.errfile  = os.path.join(os.getcwd(),'err.pyConf') 
            self.textBeforeMake = self.input.text.get('1.0', END+'-1c') 
            os.system('echo "Made with pyConf" >'+self.makefile) 
            makeScript = open('makeScript.pyConf','w')
            makeScript.write('make >>'+self.makefile+' 2>'+self.errfile+';\n')
            makeScript.write('echo $?>'+os.path.join(os.getcwd(),'make_return.pyConf'))
            makeScript.close()
            os.system("'' >"+os.path.join(os.getcwd(),'make_return.pyConf'))
            os.system('chmod u+x '+os.path.join(os.getcwd(),'makeScript.pyConf')+' 2>'+self.configerrfile)
            os.system(''+os.path.join(os.getcwd(),'makeScript.pyConf')+' 2>'+self.errfile+'&')
            self.widget.config(cursor='watch')
            self.onDoCheckMake() 
        except: 
            sys.exit(1) 
 
    def onDoCheckMake(self): 
        tmpfile = os.path.join(os.getcwd(),'checkmake.pyConf') 
        text = open(self.makefile,'r').read() 
        open(tmpfile,'w').write(self.textBeforeMake) 
        open(tmpfile,'a').write(text) 
        self.input.settext(open(tmpfile,'r').read()) 
        self.input.text.see(END) 
        self.main.update() 
        os.remove(tmpfile) 
 
        text=[] 
        text=open(self.makefile,'r').readlines() 
 
        bool1 = 1 
        bool2 = None 

        os.system("grep 'MOLCAS'  make.pyConf | wc -l > noentallbare_omhundreeralltingglemt.txt")
        nr =float(open('noentallbare_omhundreeralltingglemt.txt','r').read())
        os.remove('noentallbare_omhundreeralltingglemt.txt')
        self.waitbar.setvar(nr, None)

        
        if not '*** Build of MOLCAS is complete\n' in text: 
            bool1 = None
            self.waitbar.setvar(nr, None)
        else:
            self.waitbar.setvar(nr, 1)
            showinfo('Installation completed.','Congratulations with your new Molcas installation!\n\nFor more information and help with Molcas, please consult http://www.teokem.lu.se/molcas')
            
 
        if open(os.path.join(os.getcwd(),'make_return.pyConf'),'r').read()!='':
            bool2=1
            for i in open(os.path.join(os.getcwd(),'make_return.pyConf'),'r').readlines(): 
                if '0' in string.split(i): 
                    bool2 = None
 
        if bool2: 
            tmpfile = os.path.join(os.getcwd(),'checkmake.pyConf') 
            text = open(self.errfile,'r').read() 
            text2 = open(self.makefile,'r').read() 
            open(tmpfile,'w').write(self.textBeforeMake) 
            open(tmpfile,'a').write(text2) 
            open(tmpfile,'a').write(text) 
            self.input.settext(open(tmpfile,'r').read()) 
            self.input.text.see(END) 
            self.main.update() 

            
            # report error 
            if askyesno('Warning!','Something went wrong when building Molcas. Do you want to exit into text-mode?'): 
                sys.exit(1) 
        else: 
            if not bool1: 
                self.main.after(1000, self.onDoCheckMake) 
 
        if bool1 or bool2: 
            os.remove(self.makefile) 
            os.remove(self.errfile) 
            self.back2Button.config(state=NORMAL) 
            self.makeButton.config(state=NORMAL) 
            self.exitButton.config(state=NORMAL) 
            self.main.bind('<Escape>', self.escp ) 

            os.remove(os.path.join(os.getcwd(),'make_return.pyConf'))
            os.remove('makeScript.pyConf')
            self.widget.config(cursor='arrow')

                                       
    def onMouseEnter(self, event): 
        self.help.config(text=event.widget.descr) 
        self.help.pack() 
         
    def onMouseLeave(self, event=None): 
        self.help.config(text='') 
        self.help.pack() 
  
    def escp(self, event=None): 
        # Exit 
        self.main.quit() 
 
    def onSlogans(self): 
        self.slogans.config(text=self.sloganList[self.slogannumber]) 
        self.slogans.pack() 
        self.slogannumber=self.slogannumber+1 
        if self.slogannumber>=len(self.sloganList): 
            self.slogannumber = 0 
        self.widget.after(10000, self.onSlogans) 
         
    def OK(self): 
        self.widget.config(cursor='watch')
        self.installButton.config(state=DISABLED) 
        self.exitButton.config(state=DISABLED) 
 
        self.main.update() 
        # programs 
        co=-1 
        for key in self.packageList: 
            co=co+1 
            if self.var[co]: 
                if not self.var[co].get(): 
                    self.doUnInstall(key) 
                    self.main.update() 
                     
        # packages 
        co=-1 
        for i in self.pvar: 
            co=co+1 
            if i.get(): 
                self.doInstall(co) 
                self.main.update() 
         
        # re-read the buttons  
        self.lowerFrame.destroy() 
        self.buttonframe.destroy() 

        self.widget.config(cursor='arrow')
        self.buildGoToBlasWidget()

         
 
    def doUnInstall(self, key): 
 
 
        tmpfile = os.path.join(os.getcwd(),'uninstall.pyConf') 
        open(tmpfile,'w').write(self.input.text.get('1.0', END+'-1c')) 
         
        os.system('sbin/uninstall '+key.name+' >>'+tmpfile) 
 
        self.input.settext(open(tmpfile,'r').read()) 
        self.input.text.see(END) 
        os.remove(tmpfile) 
         
    def doInstall(self, co): 
         
        tmpfile = os.path.join(os.getcwd(),'install.pyConf') 
        open(tmpfile,'w').write(self.input.text.get('1.0', END+'-1c')) 
 
        # packages 
        os.system('sbin/install '+self.aList[co]+' >>'+tmpfile) 
 
        self.input.settext(open(tmpfile,'r').read()) 
        self.input.text.see(END) 
        os.remove(tmpfile) 
 
#################################### 
# New functions                    # 
#################################### 
 
def findDef(): 
    aList = [] 
    anotherList = [] 
     
    for file in glob.glob("src/*/*.def"): 
        a = open(file,'r').readlines() 
        for i in a: 
            str = string.lower(i) 
            bool1 = '# guess=hide\n'== str 
            bool2 = '# guess=yes\n' == str 
            bool3 = '# guess=no\n'  == str 
            if bool1 or bool2 or bool3: 
                b = os.path.split(file) 
                c = string.split(b[-1],'.') 
                aList.append( Code(name=c[0], guess=(string.strip(string.lower(i))) ) ) 
                anotherList.append(c[0]) 
 
            if '#%' in string.split(i): 
                aList[-1].descr = i[2:-1] 
 
                     
    for i in aList: 
        if not i.descr: i.descr='No description found' 
 
    anotherList.sort() 
 
    theLastList = [] 
    for i in anotherList: 
        for j in aList: 
            if i==j.name: 
                theLastList.append(j) 
                 
    return theLastList 
 
def findPackages(): 
    aList=[] 
    for file in glob.glob("src/*/*PACKAGE"):
        b = os.path.split(file)
       
        c = string.split(b[0],'/')[-1]
        aList.append(c) 
 
    anotherList=[] 
    for i in aList: 
        anotherList.append(i) 
    anotherList.sort() 
 
    return anotherList 
 
################################### 
 
if __name__=='__main__': 
    try: 
        import Tkinter    
    except: 
        sys.exit(1) 
     
    test = tkMolcasConfig() 
    test.main.mainloop() 
  
    sys.exit(0) 
