#! /usr/bin/env perl
# $ this file belongs to the Molcas repository $

# Program that converts input data for SEWARD into XYZ format
# It outputs the LAST encountered coordinate set!

use 5.004;
use strict;
use POSIX;

my $BOHR2ANGS=0.5292;
my $ZERO_DIST=1E-3;

die <<"~~DESCR~~" if @ARGV && $ARGV[0] eq '--what';
Convert SEWARD input to XYZ file.
~~DESCR~~

die <<"~~HELP~~" if @ARGV<2 || $ARGV[0] eq '--help';
Convert SEWARD input to XYZ file.
Usage:
 $0 input output.xyz
If input is "-" it reads standard input.
If output is "-" it writes to standard output.
~~HELP~~

my $arg_count=0;
my $gv=0;
if ($ARGV[0] eq '-gv') {
$arg_count++;
$gv++;
}
open(IN,"<$ARGV[$arg_count]") || die "Can't open '$ARGV[0]': $!\n";
open(OUT,">$ARGV[$arg_count+1]") || die "Can't open '$ARGV[1]': $!\n";

my $sym='';
my @atoms=();
while (<IN>) {
  next if /^(\*|!)/;
  s/\t/ /g;
  next if ($_=~/^\n/);
  if (my $flag=(m/^ ?&(SEWARD|GATEWAY)/i ... m/^ ?(End of input|&)/i)) {
 if(m/^ ?coord/i)
 {
   die "Input contains XYZ coordinates already!\n";
 
 }
    if ($flag==1) {
      @atoms=();
    }
    if (m/SYMM/i) {
      $sym=<IN>;
      next;
    }
    if (m/^ ?Basi/i) {
     my @at=ReadBasis(\*IN);
     die "Wrong format.\nStopped" unless @at;
     push @atoms,@at;
     next;
    }
    if (m/^ ?Xfie/i) {
     my @at=ReadXfield(\*IN);
     die "Wrong format.\nStopped" unless @at;
     push @atoms,@at;
     next;    
    } 
  } # end SEWARD
} # end input 
die "No atoms in input.\n" unless @atoms;
if ($sym) {
  my @sym_gen=split ' ',$sym;
  for my $op (@sym_gen) {
    ApplySym(\@atoms,$op);
  }
}
# original algorithm for ApplySym (galex) is clearly faulty.
# let's make a hack.
my $na=scalar(@atoms);
my @mult;
my $j;
my $i=0;
my $ima;
my @ATOMS;
my @UATOMS;
my $ui=0;
for(@atoms)
{

  ($ATOMS[$i][0],$ATOMS[$i][1] ,$ATOMS[$i][2], $ATOMS[$i][3])=@$_;
   $i++;
}
  
  $na=$i;
 for ($i=0; $i<$na; $i++)
 {
  
  $ima=0;
  
 for ($j=$i+1; $j<$na; $j++)
 {
	 
	 if( abs($ATOMS[$i][1]-$ATOMS[$j][1])<$ZERO_DIST && 
	     abs($ATOMS[$i][2]-$ATOMS[$j][2])<$ZERO_DIST && 
	     abs($ATOMS[$i][3]-$ATOMS[$j][3])<$ZERO_DIST)
	 {
	   $ima++;
	 }
 }
  if($ima==0) { $UATOMS[$ui][0]=$ATOMS[$i][0]; 
                $UATOMS[$ui][1]=$ATOMS[$i][1]; 
		$UATOMS[$ui][2]=$ATOMS[$i][2];
		$UATOMS[$ui][3]=$ATOMS[$i][3];   $ui++;}

} 

$na=$ui;

if($gv==0)
{
print OUT ($na,"\n");
print OUT "\n";
for($i=0;$i<$na; $i++)
{
  printf OUT "%3s %14.5f %14.5f %14.5f\n",
  $UATOMS[$i][0],$UATOMS[$i][1],$UATOMS[$i][2],$UATOMS[$i][3];

}

#for (@atoms) {
#  printf OUT "%3s %14.5f %14.5f %14.5f\n",@$_ ;
#}
}
else
{
print 'not implemented';

}

# ---------------------------------------------------------
# ---------------------------------------------------------


sub isNumber # is the argument a number?
{
 local $!=0;
 return undef if !defined($_[0]) or  $_[0] eq '';
 
 my ($v,$n_unparsed)=POSIX::strtod($_[0]);
 return undef if ($n_unparsed!=0 || $!);
 return ($v==0)?'0.E0':$v;
}

# ---------------------------------------------------------
# Reads basis section (atoms and coords) from the given FH
# Returns list of array-refs: ([$symbol, $x, $y, $z],...)
sub ReadBasis
{
 my $fh=shift;
 my $ln=0;
 my $basis='';
 my @atoms=();
 my $l;
 
 while (<$fh>) {
   $ln++;
   if (m/^End of basis/i) {
     last;
   }
   chomp;
#   next if($_=~/^ *\*/);
   
   if ($ln==1) {
     $basis=$_;
     if ($_=~/inline/i)
      {
       while(<$fh>)
        {
         last if ($_=~/^[a-zA-Z]/);
        }
     chomp;
      }
      else 
      {    
        next;
      }
   }
  next if /^(\*|!)/;
  next if ($_=~/^ *\n/);
  next if($_ eq '');
   s/,/ /g; # allow commas in input
   my ($label,$x,$y,$z,$units)=split ' ',$_,5;
#   $label=~s/\W//;
   if($label eq '')
    {
      die "check the input, line: $_\n";
    }
   unless (isNumber($z) && isNumber($y) && isNumber($x)) { # it's not a coord
     next;
   }
   unless ($units && $units=~/angs/i) {
     foreach ($x,$y,$z) {
       $_*=$BOHR2ANGS;
     }
   }
   $basis=$label;
   $basis=~s/\..*//;
   
   push @atoms,[ucfirst($basis),$x,$y,$z];
   
 }
 return @atoms;
}
# ---------------------------------------------------------
# Reads basis section (atoms and coords) from the given FH
# Returns list of array-refs: ([$symbol, $x, $y, $z],...)
sub ReadXfield
{
 my $fh=shift;
 my $ln=0;
 my $basis='';
 my @atoms=();
 my $l;
 my $n;
 my $i;
 $n=<$fh>;
 for ($i=0; $i<$n; $i++)
 {
     $l=<$fh>;
     chomp($l);
     s/,/ /g; # allow commas in input
     my( $x, $y, $z, $q, $p1, $p2, $p3, $units )=split ' ',$l,8;
   unless ($units && $units=~/angs/i) {
     foreach ($x,$y,$z) {
       $_*=$BOHR2ANGS;
     }
   }
   $basis='Qn' if($q<0);
   $basis='Qp' if($q>0);
   push @atoms,[$basis,$x,$y,$z];
 
 }
   
 return @atoms;
}


# ---------------------------------------------------------
# Applies symmetry generator
# In: ref to array of atoms, symbol of generator
# Adds new atoms to array
sub ApplySym
{
 my $atoms=shift;
 my $op=shift;
 
 # convert symm. generator into list of affected indexes (1 and/or 2 and/or 3)
 my @idx=map {
     my $i=index('xyz',$_)+1;
     die "Wrong symmetry generator: $_\nStopped" if ($i==0);
     $i;
   } split //,lc($op);
 
 my @new_atoms=();
 for my $at (@$atoms) {
   my @new_at=@$at;
   my $is_affected=0;
   for my $i (@idx) {
     if (abs($new_at[$i])>$ZERO_DIST) { # is this coord is affected?
       $is_affected=1;
       $new_at[$i]=-$new_at[$i];
     }
   }
   push @new_atoms,[@new_at] if ($is_affected); # new atom generated -- add it
 }
 push @$atoms,@new_atoms; # add all newly generated atoms
}
