#!/bin/bash
if [ $# -eq 1 ]; then
  option=$1
  echo $1 | grep '\-\-local-molcas-version=' 
  if  [ $? -eq 0 ]; then
     molcasdir=`echo $option | sed -e 's/^.*=//' `
     if [ ! -d $molcasdir ]; then
      echo "$molcasdir is not a directory" 
      exit; 
     fi 
      echo "local molcas directory:$molcasdir "
      if [ ! -f "$molcasdir/.molcasversion" ]; then 
        if [ ! -f "$molcasdir/.molcashome" ] ; then
       echo "$molcasdir is not a valid molcas directory"
       exit;
        fi 
      fi 
  else
     echo "usage:  ./usersetup.sh [--local-molcas-version=<absolute path to installed Molcas]"
     exit
  fi
# it is necessary to add the dynamic loader, to avoid incompatibilities
# with OLDER version; note, that all executables have been appropriately
# modified to use this loader at this location 
  cp syslibs/ld-linux-x86-64.so.2  /tmp/
  perl ./install_dist.pl $molcasdir | tee install_dist.log 
else 
  if [ ! -f "license.dat" ]; then 
    echo "supply molcas license file (license.dat) in current directory "
    exit 99
  fi
  cp license.dat  MOLCAS
  cp syslibs/ld-linux-x86-64.so.2  /tmp/
  perl ./install_dist.pl | tee install_dist.log 
fi 

echo "proceed by "
echo "
      1. make sure, that you can log onto your  
         system password free  via 
          'ssh  $USER@$HOSTNAME '
      2. running the test suite with this setup    
         $PWD/runtests.sh  
      3. resetting the MOLCAS environment to point to your native MOLCAS installation    
         i.e. edit $PWD/COLUMBUS/runcolmol  
      4. run the test suite with your native MOLCAS installation  
         $PWD/runtests.sh  
      5. for usage cf. to examples $PWD/TESTS and documentation  
         $PWD/COLUMBUS/docs " 
echo 
echo "Ready to use:"
echo "usage: $PWD/COLUMBUS/runcolmol <projectname> "
echo "inputfile = <projectname>.input "
echo "outputfile = <projectname>.output "

